# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fintekkers/requests/price/create_price_response.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fintekkers.models.price import price_pb2 as fintekkers_dot_models_dot_price_dot_price__pb2
from fintekkers.requests.price import create_price_request_pb2 as fintekkers_dot_requests_dot_price_dot_create__price__request__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5fintekkers/requests/price/create_price_response.proto\x12\x19\x66intekkers.requests.price\x1a#fintekkers/models/price/price.proto\x1a\x34\x66intekkers/requests/price/create_price_request.proto\"\xd0\x01\n\x18\x43reatePriceResponseProto\x12\x14\n\x0cobject_class\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12P\n\x14\x63reate_price_request\x18\x14 \x01(\x0b\x32\x32.fintekkers.requests.price.CreatePriceRequestProto\x12;\n\x0eprice_response\x18\x1e \x03(\x0b\x32#.fintekkers.models.price.PriceProtoB\x1d\x42\x19\x43reatePriceResponseProtosP\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.requests.price.create_price_response_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'B\031CreatePriceResponseProtosP\001'
  _CREATEPRICERESPONSEPROTO._serialized_start=176
  _CREATEPRICERESPONSEPROTO._serialized_end=384
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fintekkers/requests/util/errors/message.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n-fintekkers/requests/util/errors/message.proto\x12\x1f\x66intekkers.requests.util.errors\"B\n\x07Message\x12\x18\n\x10message_for_user\x18\x01 \x01(\t\x12\x1d\n\x15message_for_developer\x18\x02 \x01(\tb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.requests.util.errors.message_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _MESSAGE._serialized_start=82
  _MESSAGE._serialized_end=148
# @@protoc_insertion_point(module_scope)



from fintekkers.requests.price import query_price_request_pb2 as _query_price_request_pb2
from fintekkers.requests.price import query_price_response_pb2 as _query_price_response_pb2
from fintekkers.requests.price import create_price_request_pb2 as _create_price_request_pb2
from fintekkers.requests.price import create_price_response_pb2 as _create_price_response_pb2
from fintekkers.requests.util.errors import summary_pb2 as _summary_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import service as _service
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class Price(_service.service): ...

class Price_Stub(Price): ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fintekkers/requests/util/operation.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n(fintekkers/requests/util/operation.proto\x12\x18\x66intekkers.requests.util*a\n\x19RequestOperationTypeProto\x12\x15\n\x11UNKNOWN_OPERATION\x10\x00\x12\x0c\n\x08VALIDATE\x10\x01\x12\n\n\x06\x43REATE\x10\x02\x12\x07\n\x03GET\x10\x03\x12\n\n\x06SEARCH\x10\x04\x42\rB\tOperationP\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.requests.util.operation_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'B\tOperationP\001'
  _REQUESTOPERATIONTYPEPROTO._serialized_start=70
  _REQUESTOPERATIONTYPEPROTO._serialized_end=167
# @@protoc_insertion_point(module_scope)



from fintekkers.models.portfolio.portfolio_pb2 import PortfolioProto

class Portfolio():
    # @staticmethod
    # def create_from():
    #     pass
    #     # SecurityProto(
    #     #     object_class="Security"
    #     # )

    def __init__(self, proto:PortfolioProto):
        self.proto:PortfolioProto = proto


    def __str__(self):
        return f"ID[{self.proto.uuid}], Portfolio[{self.proto.portfolio_name}]"
    
    def get_name(self):
        return self.proto.portfolio_name
    
    # def uuid(self) -> UUID:
    #     uuid:FintekkersUuid = ProtoSerializationUtil.deserialize(self.proto.uuid)
    #     return uuid.uuid
        


# class Security(RawDataModelObject, IFinancialModelObject):
#     def __init__(self, id: uuid.UUID, issuer: str, as_of: str, settlement_currency):
#         super().__init__(id, as_of)
#         self.issuer = issuer
#         self.settlement_currency = settlement_currency
#         self.identifier = None
#         self.product_type = None
#         self.description = None

#     def get_settlement_currency(self):
#         return self.settlement_currency

#     def is_cash(self):
#         return False

#     def get_issuer(self):
#         return self.issuer

#     def get_asset_class(self):
#         return 'Unclassified'

#     def get_quantity_type(self) -> SecurityQuantityTypeProto:
#         return SecurityQuantityTypeProto.UNKNOWN_QUANTITY_TYPE

#     def get_security_id(self):
#         return self.identifier

#     def set_security_id(self, identifier):
#         self.identifier = identifier

#     def get_product_class(self):
#         return type(self).__name__

#     def get_product_type(self):
#         return SecurityTypeProto.SecurityTypeProto

#     def set_product_type(self, product_type):
#         self.product_type = product_type

#     def get_fields(self) -> Set[FieldProto]:
#         return {FieldProto.ID, FieldProto.ASSET_CLASS, FieldProto.PRODUCT_CLASS}

#     def get_measure(self, measure: MeasureProto):
#         raise NotImplementedError

#     def get_measures(self) -> Set[MeasureProto]:
#         raise NotImplementedError

#     def get_security_type(self):
#         raise RuntimeError('Not supported. Need to code this in')

#     def get_display_description(self):
#         if self.description is not None:
#             return self.description
#         elif self.identifier is not None:
#             return str(self.identifier)
#         else:
#             return str(self)

#     def get_description(self):
#         return self.description

#     def set_description(self, description):
#         self.description = description

#     def __str__(self):
#         return f'ID[{self.get_id().hex}], {type(self).__name__}[{self.issuer}]'

#     def __eq__(self, other):
#         if isinstance(other, Security):
#             return self.get_id() == other.get_id()
#         else:
#             return False

#     def __lt__(self, other):
#         if isinstance(other, Security):
#             return self.get_id() < other.get_id()
#         else:
#             return False

#     def __hash__(self):
#         return hash(self.get_id())


# class CashSecurity(Security):
#     pass



from uuid import UUID, uuid4

class FintekkersUuid:
  uuid:UUID

  def __init__(self, uuid:UUID):
    self.uuid = uuid

  def __str__(self) -> str:
    return self.uuid.__str__()

  @staticmethod
  def from_uuid(uuid:UUID):
    return FintekkersUuid(uuid)

  @staticmethod
  def from_bytes(raw_uuid:list[bytes]):
    return FintekkersUuid(UUID(bytes=raw_uuid))

  @staticmethod
  def new_uuid():
    return FintekkersUuid(uuid4())

  def uuid(self) -> UUID:
    return self.uuid
      
  def bytes(self) -> list[bytes]:
    return self.uuid.bytes




# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fintekkers/requests/util/errors/error.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fintekkers.requests.util.errors import message_pb2 as fintekkers_dot_requests_dot_util_dot_errors_dot_message__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+fintekkers/requests/util/errors/error.proto\x12\x1f\x66intekkers.requests.util.errors\x1a-fintekkers/requests/util/errors/message.proto\"\x80\x01\n\nErrorProto\x12\x38\n\x04\x63ode\x18\x01 \x01(\x0e\x32*.fintekkers.requests.util.errors.ErrorCode\x12\x38\n\x06\x64\x65tail\x18\x02 \x01(\x0b\x32(.fintekkers.requests.util.errors.Message\"\x82\x01\n\x0cWarningProto\x12\x38\n\x04\x63ode\x18\x01 \x01(\x0e\x32*.fintekkers.requests.util.errors.ErrorCode\x12\x38\n\x06\x64\x65tail\x18\x02 \x01(\x0b\x32(.fintekkers.requests.util.errors.Message*+\n\tErrorCode\x12\x11\n\rUNKNOWN_ERROR\x10\x00\x12\x0b\n\x07WARNING\x10\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.requests.util.errors.error_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _ERRORCODE._serialized_start=391
  _ERRORCODE._serialized_end=434
  _ERRORPROTO._serialized_start=128
  _ERRORPROTO._serialized_end=256
  _WARNINGPROTO._serialized_start=259
  _WARNINGPROTO._serialized_end=389
# @@protoc_insertion_point(module_scope)



from fintekkers.requests.transaction import create_transaction_request_pb2 as _create_transaction_request_pb2
from fintekkers.requests.transaction import create_transaction_response_pb2 as _create_transaction_response_pb2
from fintekkers.requests.transaction import query_transaction_request_pb2 as _query_transaction_request_pb2
from fintekkers.requests.transaction import query_transaction_response_pb2 as _query_transaction_response_pb2
from fintekkers.requests.util.errors import summary_pb2 as _summary_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import service as _service
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class Transaction(_service.service): ...

class Transaction_Stub(Transaction): ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fintekkers/requests/util/errors/summary.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fintekkers.requests.util.errors import error_pb2 as fintekkers_dot_requests_dot_util_dot_errors_dot_error__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n-fintekkers/requests/util/errors/summary.proto\x12\x1f\x66intekkers.requests.util.errors\x1a+fintekkers/requests/util/errors/error.proto\"\x8c\x01\n\x0cSummaryProto\x12;\n\x06\x65rrors\x18\x01 \x03(\x0b\x32+.fintekkers.requests.util.errors.ErrorProto\x12?\n\x08warnings\x18\x02 \x03(\x0b\x32-.fintekkers.requests.util.errors.WarningProtob\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.requests.util.errors.summary_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _SUMMARYPROTO._serialized_start=128
  _SUMMARYPROTO._serialized_end=268
# @@protoc_insertion_point(module_scope)



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class LocalTimestampProto(_message.Message):
    __slots__ = ["time_zone", "timestamp"]
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    TIME_ZONE_FIELD_NUMBER: _ClassVar[int]
    time_zone: str
    timestamp: _timestamp_pb2.Timestamp
    def __init__(self, timestamp: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., time_zone: _Optional[str] = ...) -> None: ...



from datetime import date
from fintekkers.models.security.bond.issuance_pb2 import IssuanceProto
from fintekkers.wrappers.models.util.serialization import ProtoSerializationUtil


class Issuance:
    def __init__(self, proto: list[IssuanceProto]):
        self.proto: list[IssuanceProto] = proto

    def sort_by_auction_announcement_date(self):
        self.proto = sorted(
            self.proto,
            key=lambda x: ProtoSerializationUtil.deserialize(
                x.auction_announcement_date
            ),
        )
        return self

    def print_auction_history(self):
        print(
            "announcement_date,mature_security_amount,total_accepted,post_auction_outstanding_quantity"
        )

        for issue in self.proto:
            issue: IssuanceProto

            auction_announcement_date: date = ProtoSerializationUtil.deserialize(
                issue.auction_announcement_date
            )
            mature_security_amount = 0.0
            total_accepted = 0.0
            post_auction_outstanding_quantity = 0.0

            if (
                issue.mature_security_amount is not None
                and issue.mature_security_amount.arbitrary_precision_value != ""
            ):
                mature_security_amount: float = ProtoSerializationUtil.deserialize(
                    issue.mature_security_amount
                )

            if (
                issue.total_accepted is not None
                and issue.total_accepted.arbitrary_precision_value != ""
            ):
                total_accepted: float = ProtoSerializationUtil.deserialize(
                    issue.total_accepted
                )

            if (
                issue.post_auction_outstanding_quantity is not None
                and issue.post_auction_outstanding_quantity.arbitrary_precision_value
                != ""
            ):
                post_auction_outstanding_quantity: float = (
                    ProtoSerializationUtil.deserialize(
                        issue.post_auction_outstanding_quantity
                    )
                )

            print(
                "{},${:,.2f},${:,.2f},${:,.2f}".format(
                    auction_announcement_date,
                    mature_security_amount,
                    total_accepted,
                    post_auction_outstanding_quantity,
                )
            )



from fintekkers.models.price import price_pb2 as _price_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class CreatePriceRequestProto(_message.Message):
    __slots__ = ["create_price_input", "object_class", "version"]
    CREATE_PRICE_INPUT_FIELD_NUMBER: _ClassVar[int]
    OBJECT_CLASS_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    create_price_input: _price_pb2.PriceProto
    object_class: str
    version: str
    def __init__(self, object_class: _Optional[str] = ..., version: _Optional[str] = ..., create_price_input: _Optional[_Union[_price_pb2.PriceProto, _Mapping]] = ...) -> None: ...



from fintekkers.requests.valuation import valuation_request_pb2 as _valuation_request_pb2
from fintekkers.requests.valuation import valuation_response_pb2 as _valuation_response_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import service as _service
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class Valuation(_service.service): ...

class Valuation_Stub(Valuation): ...



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from fintekkers.requests.position import query_position_request_pb2 as fintekkers_dot_requests_dot_position_dot_query__position__request__pb2
from fintekkers.requests.position import query_position_response_pb2 as fintekkers_dot_requests_dot_position_dot_query__position__response__pb2
from fintekkers.requests.util.errors import summary_pb2 as fintekkers_dot_requests_dot_util_dot_errors_dot_summary__pb2


class PositionStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Search = channel.unary_stream(
                '/fintekkers.services.position_service.Position/Search',
                request_serializer=fintekkers_dot_requests_dot_position_dot_query__position__request__pb2.QueryPositionRequestProto.SerializeToString,
                response_deserializer=fintekkers_dot_requests_dot_position_dot_query__position__response__pb2.QueryPositionResponseProto.FromString,
                )
        self.ValidateQueryRequest = channel.unary_unary(
                '/fintekkers.services.position_service.Position/ValidateQueryRequest',
                request_serializer=fintekkers_dot_requests_dot_position_dot_query__position__request__pb2.QueryPositionRequestProto.SerializeToString,
                response_deserializer=fintekkers_dot_requests_dot_util_dot_errors_dot_summary__pb2.SummaryProto.FromString,
                )


class PositionServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Search(self, request, context):
        """rpc GetByIds (position.QueryPositionRequestProto) returns (position.QueryPositionResponseProto);
        rpc ListIds (transaction.QueryTransactionRequestProto) returns (transaction.QueryTransactionResponseProto);
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ValidateQueryRequest(self, request, context):
        """rpc ValidateCreateOrUpdate (transaction.CreateTransactionRequestProto) returns (util.errors.SummaryProto);
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_PositionServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Search': grpc.unary_stream_rpc_method_handler(
                    servicer.Search,
                    request_deserializer=fintekkers_dot_requests_dot_position_dot_query__position__request__pb2.QueryPositionRequestProto.FromString,
                    response_serializer=fintekkers_dot_requests_dot_position_dot_query__position__response__pb2.QueryPositionResponseProto.SerializeToString,
            ),
            'ValidateQueryRequest': grpc.unary_unary_rpc_method_handler(
                    servicer.ValidateQueryRequest,
                    request_deserializer=fintekkers_dot_requests_dot_position_dot_query__position__request__pb2.QueryPositionRequestProto.FromString,
                    response_serializer=fintekkers_dot_requests_dot_util_dot_errors_dot_summary__pb2.SummaryProto.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'fintekkers.services.position_service.Position', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Position(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Search(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/fintekkers.services.position_service.Position/Search',
            fintekkers_dot_requests_dot_position_dot_query__position__request__pb2.QueryPositionRequestProto.SerializeToString,
            fintekkers_dot_requests_dot_position_dot_query__position__response__pb2.QueryPositionResponseProto.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ValidateQueryRequest(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fintekkers.services.position_service.Position/ValidateQueryRequest',
            fintekkers_dot_requests_dot_position_dot_query__position__request__pb2.QueryPositionRequestProto.SerializeToString,
            fintekkers_dot_requests_dot_util_dot_errors_dot_summary__pb2.SummaryProto.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from fintekkers.models.security import security_pb2 as _security_pb2
from fintekkers.requests.security import query_security_request_pb2 as _query_security_request_pb2
from fintekkers.requests.util.errors import summary_pb2 as _summary_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class QuerySecurityResponseProto(_message.Message):
    __slots__ = ["errors_or_warnings", "object_class", "query_security_input", "security_response", "version"]
    ERRORS_OR_WARNINGS_FIELD_NUMBER: _ClassVar[int]
    OBJECT_CLASS_FIELD_NUMBER: _ClassVar[int]
    QUERY_SECURITY_INPUT_FIELD_NUMBER: _ClassVar[int]
    SECURITY_RESPONSE_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    errors_or_warnings: _containers.RepeatedCompositeFieldContainer[_summary_pb2.SummaryProto]
    object_class: str
    query_security_input: _query_security_request_pb2.QuerySecurityRequestProto
    security_response: _containers.RepeatedCompositeFieldContainer[_security_pb2.SecurityProto]
    version: str
    def __init__(self, object_class: _Optional[str] = ..., version: _Optional[str] = ..., query_security_input: _Optional[_Union[_query_security_request_pb2.QuerySecurityRequestProto, _Mapping]] = ..., security_response: _Optional[_Iterable[_Union[_security_pb2.SecurityProto, _Mapping]]] = ..., errors_or_warnings: _Optional[_Iterable[_Union[_summary_pb2.SummaryProto, _Mapping]]] = ...) -> None: ...



import grpc
from typing import Dict
from pathlib import Path

SERVICE_MAP: Dict[str, str] = {
    "SecurityService": "34.205.69.2",
    "LedgerService": "34.205.69.2",
    "TransactionService": "34.205.69.2",
    "PortfolioService": "34.205.69.2",
    "ValuationService": "54.165.101.211"
}

PORT_MAP: Dict[str, str] = {
    "SecurityService": "8082",
    "LedgerService": "8082",
    "TransactionService": "8082",
    "PortfolioService": "8082",
    "ValuationService": "8080"
}

LEDGER_SERVICE = "LedgerService"
VALUATION_SERVICE = "ValuationService"
SECURITY_SERVICE = "SecurityService"
TRANSACTION_SERVICE = "TransactionService"
PORTFOLIO_SERVICE = "PortfolioService"

class Environment(object):
    IS_RUNNING_LOCALLY = True

    def __new__(cls):
        if not hasattr(cls, 'instance'):
            cls.instance = super(Environment, cls).__new__(cls)
        return cls.instance

    def __get_base_url(self, service: str) -> str:
        if Environment().IS_RUNNING_LOCALLY:
            return "host.docker.internal" if Path("/.dockerenv").exists() else "127.0.0.1"

        if service in SERVICE_MAP:
            return SERVICE_MAP[service]
        else:
            raise ValueError("Could not get location of service")

    def get_endpoint(self, service: str) -> str:
        # base_url = "http://" + __get_base_url(service)
        base_url = self.__get_base_url(service)

        port = PORT_MAP.get(service, None)
        if port is None:
            raise ValueError("Could not get port")

        return base_url + ":" + port

    def get_service_insecure_channel(self, service_name:str) -> grpc.Channel:
        '''
            Creates a HTTP channel for sending/receiving GRPC requests

                Parameters:
                        The name of the services. Note, these are currently defined in language by language
                        implementations and are manually kept in sync.

                Returns:
                        channel (grpc.Channel): Returns an insecure/HTTP channel for GRPC comms. 
        '''
        return grpc.insecure_channel(self.get_endpoint(service_name))
    

    def get_service_secure_channel(self, service_name:str) -> grpc.Channel:
        '''
            Creates a HTTPS channel for sending/receiving GRPC requests, using default locations to search for
            private keys

                Parameters:
                        The name of the services. Note, these are currently defined in language by language
                        implementations and are manually kept in sync.

                Returns:
                        channel (grpc.Channel): Returns a one-way secure/HTTPS channel for GRPC comms. Two way SSL 
                        not currently implemented.
        '''
        return grpc.secure_channel(self.get_endpoint(service_name), grpc.ssl_channel_credentials())


from typing import Generator

from google.protobuf.timestamp_pb2 import Timestamp
from google.protobuf.any_pb2 import Any
from google.protobuf import wrappers_pb2 as wrappers

from datetime import datetime
from uuid import uuid4

from fintekkers.models.portfolio.portfolio_pb2 import PortfolioProto
from fintekkers.models.position.position_filter_pb2 import PositionFilterProto
from fintekkers.models.position.position_util_pb2 import FieldMapEntry
from fintekkers.models.position import field_pb2
from fintekkers.models.util.uuid_pb2 import UUIDProto
from fintekkers.models.util.local_timestamp_pb2 import LocalTimestampProto
from fintekkers.requests.portfolio.create_portfolio_request_pb2 import CreatePortfolioRequestProto
from fintekkers.requests.portfolio.create_portfolio_response_pb2 import CreatePortfolioResponseProto

from fintekkers.requests.portfolio.query_portfolio_request_pb2 import QueryPortfolioRequestProto
from fintekkers.requests.portfolio.query_portfolio_response_pb2 import QueryPortfolioResponseProto

from fintekkers.services.portfolio_service.portfolio_service_pb2_grpc import PortfolioStub

from fintekkers.wrappers.models.portfolio import Portfolio
from fintekkers.wrappers.models.util.serialization import ProtoSerializationUtil
from fintekkers.wrappers.services.util.Environment import get_channel

class PortfolioService():
    def __init__(self):
        self.stub = PortfolioStub(get_channel())

    def search(self, request:QueryPortfolioRequestProto) -> Generator[Portfolio, None, None]:
        responses = self.stub.Search(request=request)

        try:
            while not responses._is_complete():
                response:QueryPortfolioResponseProto = responses.next()
                
                for portfolio_proto in response.portfolio_response:
                    yield Portfolio(portfolio_proto)
        except StopIteration:
            pass
        except Exception as e:
            print(e)
        
        #This will send the cancel message to the server to kill the connection
        responses.cancel()

    def create_or_update(self, request:CreatePortfolioRequestProto) -> Generator[Portfolio, None, None]:
        response:CreatePortfolioResponseProto = self.stub.CreateOrUpdate(request=request)
        yield Portfolio(response.portfolio_response)

    def create_portfolio(self, portfolio_name:str):
        uuid_value = uuid4()
        create_portfolio_request = CreatePortfolioRequestProto(
            create_portfolio_input=PortfolioProto(
                as_of= LocalTimestampProto(
                    timestamp=Timestamp(), 
                    time_zone="America/New_York"),
                is_link=False,
                object_class="Portfolio",
                portfolio_name=portfolio_name,
                uuid= UUIDProto(raw_uuid=uuid_value.bytes),
                version='0.0.1'
            )
        )
        
        responses = self.create_or_update(create_portfolio_request)

        response = None
        for _response in responses:
            response = _response

        return response

    def get_or_create_portfolio(self, portfolio_name:str):
        def wrap_string_to_any(my_string:str):
            my_any = Any()
            my_any.Pack(wrappers.StringValue(value=my_string))
            return my_any
            

        as_of_proto:LocalTimestampProto = ProtoSerializationUtil.serialize(datetime.now())

        portfolio_query = QueryPortfolioRequestProto(
            search_portfolio_input=PositionFilterProto(
                filters=[
                    FieldMapEntry(
                        field=field_pb2.FieldProto.PORTFOLIO_NAME, field_value_packed=wrap_string_to_any(portfolio_name)
                    )
                ]
            ), 
            as_of=as_of_proto
        )

        responses = self.search(portfolio_query)
        portfolios:list[Portfolio] = []
        
        for portfolio in responses:
            portfolios.append(portfolio)

        number_found = len(portfolios)
        
        if number_found == 0:
            return self.create_portfolio(portfolio_name=portfolio_name)
        else:
            return portfolios[0]



from fintekkers.models.position.field_pb2 import FieldProto
from fintekkers.models.position.measure_pb2 import MeasureProto
from fintekkers.models.position.position_pb2 import PositionProto
from fintekkers.models.position.position_util_pb2 import FieldMapEntry, MeasureMapEntry

from fintekkers.models.portfolio.portfolio_pb2 import PortfolioProto
from fintekkers.models.security.security_pb2 import SecurityProto
from fintekkers.models.security.identifier.identifier_pb2 import IdentifierProto

from fintekkers.models.util.local_date_pb2 import LocalDateProto
from fintekkers.models.util.local_timestamp_pb2 import LocalTimestampProto
from fintekkers.models.util.uuid_pb2 import UUIDProto
from fintekkers.wrappers.models.portfolio import Portfolio

from fintekkers.wrappers.models.security import Security
from fintekkers.wrappers.models.util.serialization import (
    ProtoSerializationUtil,
    ProtoEnum,
)

from google.protobuf import wrappers_pb2 as wrappers
from google.protobuf.any_pb2 import Any

from decimal import Decimal
from io import StringIO


class Position:
    positionProto: PositionProto

    def __init__(self, positionProto: PositionProto) -> None:
        self.positionProto = positionProto

    def get_field_value(self, field: FieldProto) -> object:
        """
        Returns the field value which could be any type, e.g. for Field.SECURITY a Security
        will be returned, but for Field.PORTFOLIO_ID a UUID would be returned.
        """
        return self.get_field(field_to_get=FieldMapEntry(field=field))

    def get_field(self, field_to_get: FieldMapEntry):
        """
        Returns the field value for the provided FieldMapEntry

        TODO: Why do we need this? The MeasureMapEntry should have the value inside it already?
        """
        tmp_field: FieldMapEntry

        # We'll iterate through the fields to make sure the requested field is in the proto.
        # If it's not then we'll raise a value error.
        for tmp_field in self.positionProto.fields:
            if tmp_field.field == field_to_get.field:
                if FieldProto.PORTFOLIO == field_to_get.field:
                    return Portfolio(Position.unpack_field(tmp_field))
                if FieldProto.SECURITY == field_to_get.field:
                    return Security(Position.unpack_field(tmp_field))

                unpacked_value = Position.unpack_field(tmp_field)

                if isinstance(unpacked_value, ProtoEnum):
                    descriptor = FieldProto.DESCRIPTOR.values_by_number[
                        field_to_get.field
                    ]
                    return ProtoEnum(descriptor, unpacked_value.enum_value)

                if (
                    isinstance(unpacked_value, str)
                    or isinstance(unpacked_value, float)
                    or isinstance(unpacked_value, int)
                ):
                    return unpacked_value

                return ProtoSerializationUtil.deserialize(unpacked_value)

        raise ValueError("Could not find field in position")

    def get_measure_value(self, measure: MeasureProto) -> Decimal:
        """
        Returns the decimal value for the measure
        """
        return self.get_measure(measure_to_get=MeasureMapEntry(measure=measure))

    def get_measure(self, measure_to_get: MeasureMapEntry) -> Decimal:
        """
        Returns the decimal for the given MeasureMapEntry

        TODO: Why do we need this? The MeasureMapEntry should have the value inside it already?
        """
        tmp_measure: MeasureMapEntry

        # We'll iterate through the measures to make sure the requested measure is in the proto.
        # If it's not then we'll raise a value error.
        for tmp_measure in self.positionProto.measures:
            if tmp_measure.measure == measure_to_get.measure:
                return ProtoSerializationUtil.deserialize(
                    Position.unpack_measure(tmp_measure)
                )

        raise ValueError("Could not find measure in position")

    def get_field_display(self, field_to_get: FieldMapEntry):
        field_value = self.get_field(field_to_get=field_to_get)
        return field_value.__str__()

    def get_measures(self) -> list[MeasureMapEntry]:
        return self.positionProto.measures

    def get_fields(self) -> list[FieldMapEntry]:
        return self.positionProto.fields

    def __str__(self):
        out: StringIO = StringIO()

        for field in self.get_fields():
            out.write(FieldProto.Name(number=field.field))
            out.write(",")
            out.write(self.get_field_display(field))
            out.write(";")

        for measure in self.get_measures():
            out.write(MeasureProto.Name(number=measure.measure))
            out.write(",")
            tmp: Decimal = self.get_measure(measure)
            out.write(str(tmp))
            out.write(";")

        return out.getvalue()

    @staticmethod
    def wrap_string_to_any(my_string: str):
        my_any = Any()
        my_any.Pack(wrappers.StringValue(value=my_string))
        return my_any

    @staticmethod
    def pack_field(field_to_pack):
        if field_to_pack.__class__ == LocalDateProto:
            my_any = Any()
            my_any.Pack(field_to_pack)
            return my_any

        # if field_to_unpack.field == FieldProto.PORTFOLIO_ID:
        #     return UUIDProto.FromString(field_to_unpack.field_value_packed.value)
        # if field_to_unpack.field == FieldProto.TRADE_DATE or field_to_unpack.field == FieldProto.SETTLEMENT_DATE \
        #     or field_to_unpack.field == FieldProto.TAX_LOT_OPEN_DATE\
        #         or field_to_unpack.field == FieldProto.TAX_LOT_CLOSE_DATE:
        #     return LocalDateProto.FromString(field_to_unpack.field_value_packed.value)
        # if field_to_unpack.field == FieldProto.IDENTIFIER:
        #     return IdentifierProto.FromString(field_to_unpack.field_value_packed.value)
        # if field_to_unpack.field == FieldProto.TRANSACTION_TYPE:
        #     name:str = FieldProto.DESCRIPTOR.values_by_number[field_to_unpack.field].name
        #     return ProtoEnum(name, field_to_unpack.enum_value)
        # if field_to_unpack.field == FieldProto.PORTFOLIO_NAME or field_to_unpack.field == FieldProto.SECURITY_DESCRIPTION \
        #     or field_to_unpack.field == FieldProto.PRODUCT_TYPE:
        #     return wrappers.StringValue.FromString(field_to_unpack.field_value_packed.value).value
        # if field_to_unpack.field == FieldProto.PORTFOLIO:
        #     return PortfolioProto.FromString(field_to_unpack.field_value_packed.value)
        # if field_to_unpack.field == FieldProto.SECURITY:
        #     return SecurityProto.FromString(field_to_unpack.field_value_packed.value)

        # raise ValueError(f"Field not found. Could not unpack {field_to_unpack.field}")

    @staticmethod
    def unpack_field(field_to_unpack: FieldMapEntry):
        if (
            field_to_unpack.field == FieldProto.PORTFOLIO_ID
            or field_to_unpack.field == FieldProto.SECURITY_ID
            or field_to_unpack.field == FieldProto.ID
        ):
            return UUIDProto.FromString(field_to_unpack.field_value_packed.value)
        if field_to_unpack.field == FieldProto.AS_OF:
            return LocalTimestampProto.FromString(
                field_to_unpack.field_value_packed.value
            )
        if (
            field_to_unpack.field == FieldProto.TRADE_DATE
            or field_to_unpack.field == FieldProto.MATURITY_DATE
            or field_to_unpack.field == FieldProto.ISSUE_DATE
            or field_to_unpack.field == FieldProto.SETTLEMENT_DATE
            or field_to_unpack.field == FieldProto.TAX_LOT_OPEN_DATE
            or field_to_unpack.field == FieldProto.TAX_LOT_CLOSE_DATE
        ):
            return LocalDateProto.FromString(field_to_unpack.field_value_packed.value)
        if field_to_unpack.field == FieldProto.IDENTIFIER:
            return IdentifierProto.FromString(field_to_unpack.field_value_packed.value)
        if (
            field_to_unpack.field == FieldProto.TRANSACTION_TYPE
            or field_to_unpack.field == FieldProto.POSITION_STATUS
        ):
            descriptor: str = FieldProto.DESCRIPTOR.values_by_number[
                field_to_unpack.field
            ]
            return ProtoEnum(descriptor, field_to_unpack.enum_value)
        if (
            field_to_unpack.field == FieldProto.PORTFOLIO_NAME
            or field_to_unpack.field == FieldProto.SECURITY_DESCRIPTION
            or field_to_unpack.field == FieldProto.PRODUCT_TYPE
            or field_to_unpack.field == FieldProto.ASSET_CLASS
        ):
            return wrappers.StringValue.FromString(
                field_to_unpack.field_value_packed.value
            ).value
        if field_to_unpack.field == FieldProto.PORTFOLIO:
            return PortfolioProto.FromString(field_to_unpack.field_value_packed.value)
        if field_to_unpack.field == FieldProto.SECURITY:
            return SecurityProto.FromString(field_to_unpack.field_value_packed.value)

        raise ValueError(
            f"Field not found. Could not unpack {FieldProto.Name(field_to_unpack.field)}"
        )

    from fintekkers.models.util.decimal_value_pb2 import DecimalValueProto

    @staticmethod
    def unpack_measure(measure_to_unpack: MeasureProto) -> DecimalValueProto:
        if measure_to_unpack.measure == MeasureProto.DIRECTED_QUANTITY:
            return measure_to_unpack.measure_decimal_value

        raise ValueError(
            f"Field not found. Could not unpack {MeasureProto.Name(measure_to_unpack.measure)}"
        )



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor
FIXED: CouponTypeProto
FLOAT: CouponTypeProto
UNKNOWN_COUPON_TYPE: CouponTypeProto
ZERO: CouponTypeProto

class CouponTypeProto(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

CREATE: RequestOperationTypeProto
DESCRIPTOR: _descriptor.FileDescriptor
GET: RequestOperationTypeProto
SEARCH: RequestOperationTypeProto
UNKNOWN_OPERATION: RequestOperationTypeProto
VALIDATE: RequestOperationTypeProto

class RequestOperationTypeProto(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from setuptools import setup, find_packages
import os 

VERSION = '0.0.0' 
DESCRIPTION = 'Fintekkers Ledger Models python package'
LONG_DESCRIPTION = 'Contains the generated python code for the ledger models protos, as well as hand-written wrapper code. See the package url to link to the proto defintions'

if 'BUILD_VERSION' in os.environ:
    print("******************************************")
    print("************OVERRIDING VERSION FROM ENVIRONMENT******************")
    print("******************************************")
    VERSION = os.environ.get('BUILD_VERSION')

setup(
    name = "fintekkers-ledger-models",
    license='MIT',
    author="David Doherty",
    author_email='dave@fintekkers.org',
    include_package_data=True,
    url='https://github.com/fintekkers/ledger-models',
    keywords='fintekkers ledger models',
        version=VERSION,
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        packages=find_packages(),
        install_requires=[], # add any additional packages that 
        # needs to be installed along with your package. Eg: 'caer'
        
        classifiers= [
            "Programming Language :: Python :: 3",
            "Operating System :: MacOS :: MacOS X",
        ]
)


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fintekkers/models/position/position_filter.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fintekkers.models.position import position_util_pb2 as fintekkers_dot_models_dot_position_dot_position__util__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n0fintekkers/models/position/position_filter.proto\x12\x1a\x66intekkers.models.position\x1a.fintekkers/models/position/position_util.proto\"x\n\x13PositionFilterProto\x12\x14\n\x0cobject_class\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12:\n\x07\x66ilters\x18\x15 \x03(\x0b\x32).fintekkers.models.position.FieldMapEntryB\x18\x42\x14PositionFilterProtosP\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.models.position.position_filter_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'B\024PositionFilterProtosP\001'
  _POSITIONFILTERPROTO._serialized_start=128
  _POSITIONFILTERPROTO._serialized_end=248
# @@protoc_insertion_point(module_scope)
